import os
from functools import lru_cache
from os import environ

import yaml
from pydantic_settings import BaseSettings
from yamlinclude import YamlIncludeConstructor

from argos.schemas.config import Config


class DefaultSettings(BaseSettings):
    app_env: str = "prod"
    database_url: str = ""
    yaml_file: str = ""


class DevSettings(DefaultSettings):
    database_url: str = "sqlite:////tmp/argos.db"
    yaml_file: str = "config.yaml"


class TestSettings(DefaultSettings):
    database_url: str = "sqlite:////tmp/test-argos.db"
    yaml_file: str = "tests/config.yaml"


class ProdSettings(DefaultSettings):
    pass


environments = {
    "dev": DevSettings,
    "prod": ProdSettings,
    "test": TestSettings,
}


@lru_cache()
def get_app_settings() -> DefaultSettings:
    app_env = environ.get("APP_ENV", "dev")
    settings = environments.get(app_env)
    return settings()


def read_yaml_config(filename):
    parsed = _load_yaml(filename)
    return Config(**parsed)


def _load_yaml(filename):
    base_dir = os.path.dirname(filename)
    YamlIncludeConstructor.add_to_loader_class(
        loader_class=yaml.FullLoader, base_dir=base_dir
    )

    with open(filename, "r") as stream:
        return yaml.load(stream, Loader=yaml.FullLoader)
